@@ -10,8 +10,10 @@
 # See license in LICENSE.txt file.
 #
 # Wiring-x86 is a Python module that lets you use Arduino like functionality
-# on an Intel® Galileo Gen2 board.
-
+# on
+#  Intel® Gaileo
+#  Intel® Gaileo Gen2
+#  Intel® Edison
 
 import datetime
 import os
@@ -27,6 +29,13 @@ HIGH = 'high'
 NONE = 'in'
 DRIVE_STRONG = 'strong'
 DRIVE_HIZ = 'hiz'
+MODE_0 = 'mode0'
+MODE_1 = 'mode1'
+MODE_2 = 'mode2'
+MODE_3 = 'mode3'
+MODE_4 = 'mode4'
+MODE_5 = 'mode5'
+ALL_MODES = (MODE_0, MODE_1, MODE_2, MODE_3, MODE_4, MODE_5)
 
 
 class GPIOBase(object):
@@ -44,6 +53,13 @@ class GPIOBase(object):
         self.exported_pwm = []
         self.enabled_pwm = {}
 
+        if self.has_pinmux():
+            self._export_pin(self.pinmux)
+            self._set_direction(self.pinmux, self.HIGH)
+
+    def has_pinmux(self):
+        return hasattr(self, 'pinmux')
+
     def pinMode(self, pin, mode):
         """Set mode to GPIO pin`.
 
@@ -68,6 +84,9 @@ class GPIOBase(object):
         if pin not in self.GPIO_MAPPING:
             return False
 
+        if self.has_pinmux():
+            self._set_direction(self.pinmux, self.LOW)
+
         mux = self._select_muxing(mode, pin)
         if mux is None:
             return False
@@ -95,6 +114,8 @@ class GPIOBase(object):
             elif value in (HIGH, LOW):
                 self._set_drive(vpin, DRIVE_STRONG)
                 self._write_value(vpin, value)
+            elif value in ALL_MODES:
+                self._muxmode(vpin, value)
 
         if mode == OUTPUT:
             self._set_direction(linux_pin, OUTPUT)
@@ -105,6 +126,9 @@ class GPIOBase(object):
         elif mode == PWM:
             self._init_pwm(pin)
 
+        if self.has_pinmux():
+            self._set_direction(self.pinmux, self.HIGH)
+
         return True
 
     def digitalWrite(self, pin, state):
@@ -238,7 +262,7 @@ class GPIOBase(object):
         elif mode == INPUT_PULLDOWN:
             return self.GPIO_MUX_INPUT_PULLDOWN[pin]
         elif mode == ANALOG_INPUT and pin in self.ADC_MAPPING:
-                return self.GPIO_MUX_ANALOG_INPUT[pin]
+            return self.GPIO_MUX_ANALOG_INPUT[pin]
         elif mode == PWM and pin in self.PWM_MAPPING:
             return self.GPIO_MUX_PWM[pin]
         return None
@@ -248,14 +272,14 @@ class GPIOBase(object):
             f = open('/sys/class/gpio/gpio%d/value' % linux_pin, 'r+')
             self.gpio_handlers[linux_pin] = f
         except:
-            print "Failed opening value file for pin %d" % linux_pin
+            print "Failed opening digital value file for pin %d" % linux_pin
 
     def _open_analog_handler(self, linux_pin, adc):
         try:
-            f = open('/sys/bus/iio/devices/iio:device0/in_voltage%d_raw' % adc, 'r+')
+            f = open('/sys/bus/iio/devices/iio:device%d/in_voltage%d_raw' % (self.adc_iio_device, adc), 'r+')
             self.gpio_handlers[linux_pin] = f
         except:
-            print "Failed opening value file for pin %d" % linux_pin
+            print "Failed opening analog value file for pin %d" % linux_pin
 
     def _write_value(self, linux_pin, state):
         value = 1
@@ -272,7 +296,7 @@ class GPIOBase(object):
 
     def _set_direction(self, linux_pin, direction):
         dirfile = '/sys/class/gpio/gpio%d/direction' % linux_pin
-        cmd = 'test -f %s && echo %s > %s' % (dirfile, direction, dirfile)
+        cmd = 'test -f %s && echo %s > %s 2>&1' % (dirfile, direction, dirfile)
         self._exec_cmd(self._set_direction.__name__, cmd)
 
     def _export_pin(self, linux_pin):
@@ -284,9 +308,14 @@ class GPIOBase(object):
         cmd = 'echo %d > /sys/class/gpio/unexport 2>&1' % linux_pin
         self._exec_cmd(self._unexport_pin.__name__, cmd)
 
+    def _muxmode(self, linux_pin, mode):
+        cmd = 'echo %s > /sys/kernel/debug/gpio_debug/gpio%d/current_pinmux' % (mode, linux_pin)
+        self._exec_cmd(self._muxmode.__name__, cmd)
+
     def _set_drive(self, linux_pin, drive):
-        cmd = 'echo %s > /sys/class/gpio/gpio%d/drive > /dev/null' % (drive, linux_pin)
-        self._exec_cmd(self._set_drive.__name__, cmd)
+        if not self.has_pinmux():
+            cmd = 'echo %s > /sys/class/gpio/gpio%d/drive > /dev/null' % (drive, linux_pin)
+            self._exec_cmd(self._set_drive.__name__, cmd)
 
     def _export_pwm(self, channel):
         self.exported_pwm.append(channel)
@@ -424,6 +453,7 @@ class GPIOGalileo(GPIOBase):
     PWM_DEFAULT_PERIOD = 5000000
 
     def __init__(self, **kwargs):
+        self.adc_iio_device = 0
         super(GPIOGalileo, self).__init__(**kwargs)
         self.pwm_periods = {}
         for pwm in self.PWM_MAPPING.keys():
@@ -614,6 +644,7 @@ class GPIOGalileoGen2(GPIOBase):
     PWM_DEFAULT_PERIOD = 5000000
 
     def __init__(self, **kwargs):
+        self.adc_iio_device = 0
         super(GPIOGalileoGen2, self).__init__(**kwargs)
         self.pwm_period = self.PWM_DEFAULT_PERIOD
         self.is_pwm_period_set = False
@@ -623,7 +654,7 @@ class GPIOGalileoGen2(GPIOBase):
         set all the PWM outputs are disabled for at least 1ms while the chip
         reconfigures itself. The PWM pin is then ignored.
         """
-        self.period = period
+        self.pwm_period = period
         cmd = 'echo %d > /sys/class/pwm/pwmchip0/device/pwm_period' % period
         self._exec_cmd(self._set_pwm_period.__name__, cmd)
 
@@ -638,3 +669,194 @@ class GPIOGalileoGen2(GPIOBase):
         if not self.is_pwm_period_set:
             self._set_pwm_period(pin, self.pwm_period)
             self.is_pwm_period_set = True
+
+
+class GPIOEdison(GPIOBase):
+
+    """Class for managing GPIO pinout on Intel®Edison board
+
+    See docs/ directory for more information.
+    """
+
+    GPIO_MAPPING = {                                                            
+        0: 130,                                                                 
+        1: 131,                                                                 
+        2: 128,                                                                 
+        3:  12,                                                                 
+        4: 129,                                                                 
+        5:  13,                                                                 
+        6: 182,                                                                 
+        7:  48,                                                                 
+        8:  49,                                                                 
+        9: 183,                                                                 
+       10:  41,                                                                 
+       11:  43,                                                                 
+       12:  42,                                                                 
+       13:  40,                                                                 
+       14:  44,                                                                 
+       15:  45,                                                                 
+       16:  46,                                                                 
+       17:  47,                                                                 
+       18:  14,                                                                 
+       19: 165,                                                                 
+    } 
+
+    ADC_MAPPING = {
+        14: 0,
+        15: 1,
+        16: 2,
+        17: 3,
+        18: 4,
+        19: 5,
+    }
+
+    PWM_MAPPING = {
+        3: 0,
+        5: 1,
+        6: 2,
+        9: 3,
+        # TODO: enable swizzler
+        10: None,
+        11: None,
+    }
+
+    GPIO_MUX_OUTPUT = {
+        0:  ((130, MODE_0), (248, HIGH), (216, HIGH)),
+        1:  ((131, MODE_0), (249, HIGH), (217, HIGH)),
+        2:  ((128, MODE_0), (250, HIGH), (218, HIGH)),
+        3:  (( 12, MODE_0), (251, HIGH), (219, HIGH)),
+        4:  ((129, MODE_0), (252, HIGH), (220, HIGH)),
+        5:  (( 13, MODE_0), (253, HIGH), (221, HIGH)),
+        6:  ((182, MODE_0), (254, HIGH), (222, HIGH)),
+        7:  (( 48, MODE_0), (255, HIGH), (223, HIGH)),
+        8:  (( 49, MODE_0), (256, HIGH), (224, HIGH)),
+        9:  ((183, MODE_0), (257, HIGH), (225, HIGH)),
+        10: (( 41, MODE_0), (258, HIGH), (226, HIGH), (240, LOW), (263, HIGH)),
+        11: (( 43, MODE_0), (259, HIGH), (227, HIGH), (241, LOW), (262, HIGH)),
+        12: (( 42, MODE_0), (260, HIGH), (228, HIGH), (242, LOW)),
+        13: (( 40, MODE_0), (261, HIGH), (229, HIGH), (243, LOW)),
+        14: (( 44, MODE_0), (232, HIGH), (208, HIGH), (200, LOW)),
+        15: (( 45, MODE_0), (233, HIGH), (209, HIGH), (201, LOW)),
+        16: (( 46, MODE_0), (234, HIGH), (210, HIGH), (202, LOW)),
+        17: (( 47, MODE_0), (235, HIGH), (211, HIGH), (203, LOW)),
+        18: (( 14, MODE_0), (236, HIGH), (212, HIGH), (204, LOW)),
+        19: ((165, MODE_0), (237, HIGH), (213, HIGH), (205, LOW)),
+    }
+
+    GPIO_MUX_INPUT = {
+        0:  ((130, MODE_0), (248, LOW), (216, NONE)),
+        1:  ((131, MODE_0), (249, LOW), (217, NONE)),
+        2:  ((128, MODE_0), (250, LOW), (218, NONE)),
+        3:  (( 12, MODE_0), (251, LOW), (219, NONE)),
+        4:  ((129, MODE_0), (252, LOW), (220, NONE)),
+        5:  (( 13, MODE_0), (253, LOW), (221, NONE)),
+        6:  ((182, MODE_0), (254, LOW), (222, NONE)),
+        7:  (( 48, MODE_0), (255, LOW), (223, NONE)),
+        8:  (( 49, MODE_0), (256, LOW), (224, NONE)),
+        9:  ((183, MODE_0), (257, LOW), (225, NONE)),
+        10: (( 41, MODE_0), (258, LOW), (226, NONE), (240, LOW), (263, HIGH)),
+        11: (( 43, MODE_0), (259, LOW), (227, NONE), (241, LOW), (262, HIGH)),
+        12: (( 42, MODE_0), (260, LOW), (228, NONE), (242, LOW)),
+        13: (( 40, MODE_0), (261, LOW), (229, NONE), (243, LOW)),
+        14: (( 44, MODE_0), (232, LOW), (208, NONE), (200, LOW)),
+        15: (( 45, MODE_0), (233, LOW), (209, NONE), (201, LOW)),
+        16: (( 46, MODE_0), (234, LOW), (210, NONE), (202, LOW)),
+        17: (( 47, MODE_0), (235, LOW), (211, NONE), (203, LOW)),
+        18: (( 14, MODE_0), (236, LOW), (212, NONE), (204, LOW)),
+        19: ((165, MODE_0), (237, LOW), (213, NONE), (205, LOW)),
+    }
+
+    GPIO_MUX_INPUT_PULLUP = {
+        0:  ((130, MODE_0), (248, LOW), (216, HIGH)),
+        1:  ((131, MODE_0), (249, LOW), (217, HIGH)),
+        2:  ((128, MODE_0), (250, LOW), (218, HIGH)),
+        3:  (( 12, MODE_0), (251, LOW), (219, HIGH)),
+        4:  ((129, MODE_0), (252, LOW), (220, HIGH)),
+        5:  (( 13, MODE_0), (253, LOW), (221, HIGH)),
+        6:  ((182, MODE_0), (254, LOW), (222, HIGH)),
+        7:  (( 48, MODE_0), (255, LOW), (223, HIGH)),
+        8:  (( 49, MODE_0), (256, LOW), (224, HIGH)),
+        9:  ((183, MODE_0), (257, LOW), (225, HIGH)),
+        10: (( 41, MODE_0), (258, LOW), (226, HIGH), (240, LOW), (263, HIGH)),
+        11: (( 43, MODE_0), (259, LOW), (227, HIGH), (241, LOW), (262, HIGH)),
+        12: (( 42, MODE_0), (260, LOW), (228, HIGH), (242, LOW)),
+        13: (( 40, MODE_0), (261, LOW), (229, HIGH), (243, LOW)),
+        14: (( 44, MODE_0), (232, LOW), (208, HIGH), (200, LOW)),
+        15: (( 45, MODE_0), (233, LOW), (209, HIGH), (201, LOW)),
+        16: (( 46, MODE_0), (234, LOW), (210, HIGH), (202, LOW)),
+        17: (( 47, MODE_0), (235, LOW), (211, HIGH), (203, LOW)),
+        18: (( 14, MODE_0), (236, LOW), (212, HIGH), (204, LOW)),
+        19: ((165, MODE_0), (237, LOW), (213, HIGH), (205, LOW)),
+    }
+
+    GPIO_MUX_INPUT_PULLDOWN = {
+        0:  ((130, MODE_0), (248, LOW), (216, LOW)),
+        1:  ((131, MODE_0), (249, LOW), (217, LOW)),
+        2:  ((128, MODE_0), (250, LOW), (218, LOW)),
+        3:  (( 12, MODE_0), (251, LOW), (219, LOW)),
+        4:  ((129, MODE_0), (252, LOW), (220, LOW)),
+        5:  (( 13, MODE_0), (253, LOW), (221, LOW)),
+        6:  ((182, MODE_0), (254, LOW), (222, LOW)),
+        7:  (( 48, MODE_0), (255, LOW), (223, LOW)),
+        8:  (( 49, MODE_0), (256, LOW), (224, LOW)),
+        9:  ((183, MODE_0), (257, LOW), (225, LOW)),
+        10: (( 41, MODE_0), (258, LOW), (226, LOW), (240, LOW), (263, HIGH)),
+        11: (( 43, MODE_0), (259, LOW), (227, LOW), (241, LOW), (262, HIGH)),
+        12: (( 42, MODE_0), (260, LOW), (228, LOW), (242, LOW)),
+        13: (( 40, MODE_0), (261, LOW), (229, LOW), (243, LOW)),
+        14: (( 44, MODE_0), (232, LOW), (208, LOW), (200, LOW)),
+        15: (( 45, MODE_0), (233, LOW), (209, LOW), (201, LOW)),
+        16: (( 46, MODE_0), (234, LOW), (210, LOW), (202, LOW)),
+        17: (( 47, MODE_0), (235, LOW), (211, LOW), (203, LOW)),
+        18: (( 14, MODE_0), (236, LOW), (212, LOW), (204, LOW)),
+        19: ((165, MODE_0), (237, LOW), (213, LOW), (205, LOW)),
+    }
+
+    GPIO_MUX_ANALOG_INPUT = {
+        14: (( 44, MODE_0), (200, HIGH), (232, LOW), (208, NONE)),
+        15: (( 45, MODE_0), (201, HIGH), (233, LOW), (209, NONE)),
+        16: (( 46, MODE_0), (202, HIGH), (234, LOW), (210, NONE)),
+        17: (( 47, MODE_0), (203, HIGH), (235, LOW), (211, NONE)),
+        18: (( 14, MODE_0), (204, HIGH), (236, LOW), (212, NONE)),
+        19: ((165, MODE_0), (205, HIGH), (237, LOW), (213, NONE)),
+    }
+
+    GPIO_MUX_PWM = {
+        3:  (( 12, MODE_1), (251, HIGH), (219, NONE)),
+        5:  (( 13, MODE_1), (253, HIGH), (221, NONE)),
+        6:  ((182, MODE_1), (254, HIGH), (222, NONE)),
+        9:  ((183, MODE_1), (257, HIGH), (225, NONE)),
+        10: (( 41, MODE_1), (258, HIGH), (226, NONE), (240, LOW), (263, HIGH)),
+        11: (( 43, MODE_1), (259, HIGH), (227, NONE), (241, LOW), (262, HIGH)),
+    }
+
+    PWM_MIN_PERIOD = 104
+    PWM_MAX_PERIOD = 218453000
+    PWM_DEFAULT_PERIOD = 2048000
+
+    def __init__(self, **kwargs):
+        self.pinmux = 214
+        self.adc_iio_device = 1
+        super(GPIOEdison, self).__init__(**kwargs)
+        self.pwm_periods = {}
+        for pin in self.PWM_MAPPING.keys():
+            self.pwm_periods[pin] = self.PWM_DEFAULT_PERIOD
+        # Set all pins into a safe state at startup.
+        for i in range(0, 20):
+            self.pinMode(i, INPUT)
+
+    def _set_pwm_period(self, pin, period):
+        self.pwm_periods[pin] = period
+        channel = self.PWM_MAPPING[pin]
+        cmd = 'echo %d > /sys/class/pwm/pwmchip0/pwm%d/period' % (period, channel)
+        self._exec_cmd(self._set_pwm_period.__name__, cmd)
+
+    def _get_pwm_period(self, pin):
+        return self.pwm_periods[pin]
+
+    def _init_pwm(self, pin):
+        pwm = self.PWM_MAPPING[pin]
+        self._export_pwm(pwm)
+        self._set_pwm_period(pin, self.pwm_periods[pin])
+        self._set_pwm_duty_cycle(pwm, 0)
+        self._enable_pwm(pwm)
